import java.util.Random;

/**
 * Implementation of a simple selection sort for a fixed length array.
 * 
 * @author (based on an example from Horstmann) 
 * @version (26/08/2011)
 */
public class SortAndSearch
{
    /**
     * Main method for SelectionSort
     */
    public static void main()
    {
        int[] a = randomIntArray(20, 100);

        print(a);                               // print the unsorted array
        sort(a);                                // sort the array
        print(a);                               // print the sorted array

        // Search an sorted array for a value. Print -1 if not found.
        int[] b = {9,11,18,20,33,47,53,54,55,55,56,61,62,62,67,69,82,84,94, 96};
        System.out.println("Found at: " + search(b, 20, 0, a.length - 1));
    }

    /**
     * Creates an array filled with random numbers
     * 
     * @param  length   the length of the array
     * @param  n        the number of possible random values
     * @return          an array filled with length numbers between 0 and n-1 
     */
    public static int[] randomIntArray(int length, int n)
    {
        int a[] = new int[length];
        Random generator = new Random();
        
        for (int i = 0; i < a.length; i++)
        {
            a[i] = generator.nextInt(n);
        }
        return a;
    }

    /**
     * Prints all elements in an array
     *
     * @param  a   the array to print
     */
    public static void print(int[] a)
    {
        for (int i = 0; i < a.length; i++)
        {
            System.out.print(a[i] + " ");
        }
        System.out.println();
    }

    /**
     * Sorts an array by the "selection sort" method.
     * Find the position of the smallest element in the array,
     * swap it with the next unsorted element
     *
     * @param  a   the array to sort
     */
    public static void sort(int[] a)
    {
        for (int n = 0; n < a.length - 1; n++)
        {
            int minPos = minimumPosition(a, n);
            
            if (minPos != n)
            {
                swap(a, minPos, n);
            }
        }
    }

    /**
     * Find the smallest element in an array range
     *
     * @param  a     the array to search
     * @param  from  the first position in a to compare
     * @return       the position of the smallest element in the range a[from] .. a[a.length - 1]
     */
    public static int minimumPosition(int[] a, int from)
    {
        int minPos = from;
        
        for (int i = from + 1; i < a.length; i++)
        {
            if (a[i] < a[minPos])
            {
                minPos = i;
            }
        }
        return minPos;
    }

    /**
     * Swaps two elements in an array
     *
     * @param  a   the array with the two elements to swap
     * @param  i   the index of one of the elements
     * @param  a   the index of the other element
     */
    public static void swap(int[] a, int i, int j)
    {
        int temp = a[i];
        a[i] = a[j];
        a[j] = temp;
    }

    /**
     * Performs a binary search for an element in an array
     *
     * @param  a     the array to be searched
     * @param  v     the value to look for
     * @param  from  the starting position in the array
     * @param  to    the end position in the array
     * @return       the position of the value, -1 if not found
     */
    public static int search(int a[], int v, int from, int to)
    {
        if (from > to)
            return -1;
 
        int mid = from + (to - from)/2;
        
        if (v == a[mid])
            return mid;
        else if (v < a[mid])
            return search(a, v, from, mid - 1);
        else
            return search(a, v, mid + 1, to);
    }
}
